import 'dart:async';

import '../entities/cell.dart';
import '../provider/data.dart';

class BoardAnimate {

  // Start game animation: blinking tiles
  static List createStartGameAnimationPatterns(Data myProvider) {
    List<List> patterns = [];

    int patternsCount = 3;
    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    for (var patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List> pattern = [];
      for (var row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (var col = 0; col < boardSideLength; col++) {
          patternRow.add(((patternIndex + row + col) % 2 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Win game animation: fill board with colored rows, from bottom to top
  static List createWinGameAnimationPatterns(Data myProvider) {
    List<List> patterns = [];

    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;
    int patternsCount = boardSideLength + 6;

    for (var patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List> pattern = [];
      for (var row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (var col = 0; col < boardSideLength; col++) {
          patternRow.add(row > (patternIndex - 4));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Default multi-purpose animation: sliding stripes, from top left to right bottom
  static List createDefaultAnimationPatterns(Data myProvider) {
    List<List> patterns = [];

    int boardSideLength = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;
    int patternsCount = boardSideLength;

    for (var patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List> pattern = [];
      for (var row = 0; row < boardSideLength; row++) {
        List<bool> patternRow = [];
        for (var col = 0; col < boardSideLength; col++) {
          patternRow.add(((patternIndex + row + col) % 4 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  static void startAnimation(Data myProvider, String animationType) {
    List patterns = [];

    switch(animationType) {
      case 'start':
        patterns = createStartGameAnimationPatterns(myProvider);
        break;
      case 'win':
        patterns = createWinGameAnimationPatterns(myProvider);
        break;
      default:
        patterns = createDefaultAnimationPatterns(myProvider);
    }

    int _patternIndex = patterns.length;

    myProvider.updateAnimationInProgress(true);

    Timer _timerAnimateBoard;
    const interval = const Duration(milliseconds: 200);
    _timerAnimateBoard = new Timer.periodic(
      interval,
      (Timer timer) {
        if (_patternIndex == 0) {
          timer.cancel();
          myProvider.resetAnimatedBackground();
          myProvider.updateAnimationInProgress(false);
        } else {
          _patternIndex--;
          myProvider.setAnimatedBackground(patterns[_patternIndex]);
        }
      },
    );
  }
}
